#!/usr/bin/env python3

import unittest
import random
from fractions import Fraction
from pypolys.polys import Poly


class TestPoly(unittest.TestCase):

    def setUp(self):
        self.x0 = Poly(1)
        self.x1 = Poly(1, 1)
        self.x2 = Poly(1, 2)
        self.x3 = Poly(1, 3)

    def test_repr(self):
        self.assertEqual(repr(Poly()), "Poly()")
        self.assertEqual(repr(Poly(0)), "Poly()")
        self.assertEqual(repr(Poly(0, 2)), "Poly()")
        self.assertEqual(repr(Poly(2, 0)), "Poly(2)")
        self.assertEqual(repr(self.x1 + self.x2), "Poly(1, 1) + Poly(1, 2)")
        self.assertEqual(repr(Poly(Fraction(3, 1), 2)), "Poly(3, 2)")
        self.assertEqual(repr(Poly(Fraction(3, 5), 2)), "Poly(Fraction(3, 5), 2)")

    def test_fromiterable(self):
        self.assertEqual(Poly.fromiterable([0, 1]), self.x1)
        self.assertEqual(Poly.fromiterable([0, 0, 1]), self.x2)
        self.assertEqual(Poly.fromiterable([0, 1, 1]), self.x1 + self.x2)

    def test_is_zero(self):
        self.assertTrue(Poly().is_zero())
        self.assertTrue(Poly(0, 2).is_zero())
        self.assertFalse(self.x1.is_zero())

    def test_degree(self):
        self.assertEqual(self.x0.degree(), 0)
        self.assertEqual(self.x1.degree(), 1)
        self.assertEqual(self.x2.degree(), 2)
        self.assertEqual(self.x3.degree(), 3)

    def test_len(self):
        self.assertEqual(len(Poly()), 0)
        self.assertEqual(len(self.x1), 1)
        self.assertEqual(len(self.x1 + self.x3), 2)

    def test_getitem(self):
        self.assertEqual(self.x2[0], 0)
        self.assertEqual(self.x2[1], 0)
        self.assertEqual(self.x2[2], 1)
        self.assertEqual(self.x2[3], 0)

    def test_add_sub(self):
        self.assertEqual(self.x2 + self.x2, Poly(2, 2))
        self.assertEqual(self.x1 - self.x1, Poly())
        self.assertEqual(self.x2 + 3, self.x2 + Poly(3))
        self.assertEqual(3 + self.x2, self.x2 + Poly(3))
        self.assertEqual(self.x2 - 3, self.x2 - Poly(3))
        self.assertEqual(3 - self.x2, Poly(3) - self.x2)

    def test_mul(self):
        self.assertEqual(self.x1 * self.x1, self.x2)
        self.assertEqual(self.x1 * self.x2, self.x3)
        self.assertEqual(self.x2 * 3, Poly(3, 2))
        self.assertEqual(3 * self.x2, Poly(3, 2))

    def test_pos_neg(self):
        self.assertEqual(+self.x1, self.x1)
        self.assertEqual(-self.x1, Poly(-1, 1))
        self.assertEqual(-self.x2, Poly(-1, 2))

    def test_combine(self):
        self.assertEqual(self.x3.combine(self.x2), Poly(1, 6))
        self.assertEqual(self.x2.combine(self.x3), Poly(1, 6))
        self.assertEqual(self.x2.combine(self.x1 + Poly(3)), 
            self.x2 + Poly(6, 1) + Poly(9))

    def test_pow(self):
        self.assertEqual(self.x1 ** 3, self.x3)
        p = self.x1 + Poly(1)
        self.assertEqual(p ** 3, p*p*p)
        self.assertEqual(p ** 5, p*p*p*p*p)
        self.assertEqual(pow(self.x1, 5).combine(p), p*p*p*p*p)
        self.assertEqual(Poly(1, 5).combine(p), p*p*p*p*p)

    def test_eval(self):
        self.assertEqual(self.x2.eval(2), 4)
        self.assertEqual(self.x3.eval(3), 27)

    def test_diff(self):
        self.assertEqual(self.x0.diff(), Poly())
        self.assertEqual(self.x1.diff(), self.x0)
        self.assertEqual(self.x3.diff(), Poly(3, 2))

    def test_integrate(self):
        self.assertEqual(Poly(2, 1).integrate(), self.x2)
        self.assertEqual(self.x1.integrate(), Poly(Fraction(1, 2), 2))
        self.assertEqual(self.x2.integrate(), Poly(Fraction(1, 3), 3))
        self.assertEqual(Poly(3, 4).integrate(), Poly(Fraction(3, 5), 5))
        self.assertEqual(Poly(Fraction(3, 2), 1).integrate(), Poly(Fraction(3, 4), 2))
        self.assertEqual(Poly(Fraction(3, 2), 1).integrate(), Poly(0.75, 2))
        self.assertEqual(Poly(1.5, 1).integrate(), Poly(0.75, 2))

    def test_call(self):
        self.assertEqual(self.x2(self.x3), Poly(1, 6))
        self.assertEqual(self.x2(2), 4)

    def test_div(self):
        self.assertEqual(self.x2 / 2.0, Poly(0.5, 2))
        self.assertEqual(self.x2 / 2, Poly(Fraction(1, 2), 2))
        self.assertEqual(self.x2 / Fraction(1, 2), Poly(2, 2))
        self.assertEqual(repr(self.x3 / 2), "Poly(Fraction(1, 2), 3)")
        self.assertEqual(repr(self.x3 / 2.0), "Poly(0.5, 3)")
        self.assertEqual(self.x3 / self.x2, self.x1)
        self.assertEqual(self.x3 / self.x3, self.x0)
        self.assertEqual(Poly(6, 5) / Poly(3, 2), Poly(2, 3))
        self.assertRaises(ValueError, lambda: self.x1 / self.x2)
        self.assertRaises(ValueError, lambda: self.x3 / (self.x2 + self.x1))
        self.assertRaises(ValueError, lambda: (self.x3 + self.x2) / self.x1)

    def test_lcm(self):
        self.assertEqual(self.x2.lcm(1), self.x2)
        self.assertEqual(self.x2.lcm(self.x3), self.x3)
        self.assertRaises(ValueError, lambda: self.x1.lcm(self.x2 + self.x3))
        self.assertRaises(ValueError, lambda: (self.x1 + self.x2).lcm(self.x3))

    def test_cancel(self):
        p1 = pow(self.x1 + 1, 2) - self.x2
        self.assertEqual(p1.degree(), 1)
        p1 = self.x3 + Poly(-1, 3)
        self.assertEqual(p1.degree(), 0)

    def test_sort(self):
        alist = [self.x0, self.x1, self.x2, self.x3]
        blist = list(alist)
        # Sortowanie wg stopni wielomianow.
        random.shuffle(blist)
        blist.sort(key=Poly.degree)
        self.assertEqual(alist, blist)
        random.shuffle(blist)
        blist.sort(key=Poly.key_lex)
        self.assertEqual(alist, blist)
        random.shuffle(blist)
        blist.sort(key=Poly.key_deglex)
        self.assertEqual(alist, blist)

    def test_leading_term(self):
        p1 = pow(3 * self.x1 + 1, 2)
        self.assertEqual(p1.leading_term(key=Poly.key_lex), 9 * self.x2)
        self.assertEqual(p1.leading_monomial(key=Poly.key_lex), self.x2)
        self.assertEqual(p1.leading_coefficient(key=Poly.key_lex), 9)
        p1 = pow(5 * self.x1 + 2, 3)
        self.assertEqual(p1.leading_term(key=Poly.key_lex), 125 * self.x3)
        self.assertEqual(p1.leading_monomial(key=Poly.key_lex), self.x3)
        self.assertEqual(p1.leading_coefficient(key=Poly.key_lex), 125)
        self.assertRaises(ValueError, Poly().leading_term, Poly.key_lex)
        self.assertRaises(ValueError, Poly().leading_monomial, Poly.key_lex)
        self.assertRaises(ValueError, Poly().leading_coefficient, Poly.key_lex)
        self.assertEqual(Poly(5).leading_term(key=Poly.key_lex), Poly(5))
        self.assertEqual(Poly(5).leading_monomial(key=Poly.key_lex), Poly(1))
        self.assertEqual(Poly(5).leading_coefficient(key=Poly.key_lex), 5)

    def test_iterterms(self):
        p1 = pow(self.x2 + 1, 2)
        self.assertEqual(len(list(p1.iterterms())), 3)
        self.assertEqual(set(term.degree() for term in p1.iterterms()), set([0, 2, 4]))
        self.assertEqual(sum(term.degree() for term in p1.iterterms()), 6)

    def tearDown(self): pass

if __name__== "__main__":

    unittest.main()

# EOF



#!/usr/bin/env python3

import random
from fractions import Fraction


class PolyFactory:
    """The class for poly generators."""

    def __init__(self, poly_class):
        """Get a poly class."""
        self.cls = poly_class
        self.HERMITE = {0: self.cls(1), 1: self.cls(2, 1)}
        self.CHEBYSHEV = {0: self.cls(1), 1: self.cls(1, 1)}
        self.LEGENDRE = {0: self.cls(1), 1: self.cls(1, 1)}

    def natural(self, n=1):
        """Create a poly (x-1)*(x-2)*...*(x-n)."""
        poly = self.cls(1)
        for i in range(1, n+1):
            poly = poly * (self.cls(1, 1) -i)
        return poly

    def geometric(self, n=1):
        """Create a poly 1 + x + x**2 + ... + x**n."""
        poly = self.cls(1)
        for i in range(1, n+1):
            poly = poly + self.cls(1, i)
        return poly

    def hermite(self, n=1):
        """Create a Hermite polynomial."""
        if n not in self.HERMITE:
            self.HERMITE[n] = (self.cls(2, 1) * self.hermite(n-1) 
                            + self.cls(2*n-2) * self.hermite(n-2))
        return self.HERMITE[n]

    def chebyshev(self, n=1):
        """Create a Chebyshev polynomial."""
        if n not in self.CHEBYSHEV:
            self.CHEBYSHEV[n] = (self.cls(2, 1) * self.chebyshev(n-1) 
                                 + self.cls(-1) * self.chebyshev(n-2))
        return self.CHEBYSHEV[n]

    def legendre(self, n=1):
        """Create a Legendre polynomial."""
        if n not in self.LEGENDRE:
            self.LEGENDRE[n] = (
                self.cls(Fraction(2*n-1, n), 1) * self.legendre(n-1) 
                   + self.cls(Fraction(-n+1, n)) * self.legendre(n-2))
        return self.LEGENDRE[n]

# EOF



#!/usr/bin/env python3

import unittest
import math
from fractions import Fraction
from pypolys.polys import Poly
#from pypolys.mpolys import Poly
from pypolys.factory import PolyFactory


class TestPolyFactory(unittest.TestCase):

    def setUp(self):
        self.N = 10
        self.poly_factory = PolyFactory(Poly)

    def test_natural(self):
        p = self.poly_factory.natural(n=self.N)
        #print(p)
        self.assertEqual(p.degree(), self.N)
        self.assertEqual(p[self.N], 1)
        self.assertEqual(p[self.N-1], sum(-i for i in range(self.N+1)))
        self.assertEqual(p[0], math.factorial(self.N))

    def test_geometric(self):
        p = self.poly_factory.geometric(n=self.N)
        #print(p)
        self.assertEqual(p.degree(), self.N)
        for i in range(self.N+1):
            self.assertEqual(p[i], 1)

    def test_hermite(self):
        p = self.poly_factory.hermite(n=self.N)
        #print p
        #print self.poly_factory.HERMITE   # sa wszystkie
        self.assertEqual(p.degree(), self.N)
        h0 = self.poly_factory.hermite(n=0)
        h1 = self.poly_factory.hermite(n=1)
        h2 = self.poly_factory.hermite(n=2)
        h3 = self.poly_factory.hermite(n=3)
        h4 = self.poly_factory.hermite(n=4)
        h5 = self.poly_factory.hermite(n=5)
        self.assertEqual(h0, Poly(1))
        self.assertEqual(h1, Poly(2, 1))
        self.assertEqual(h2, Poly(2) + Poly(4, 2))
        self.assertEqual(h3, Poly(12, 1) + Poly(8, 3))
        self.assertEqual(h4, Poly(12) + Poly(48, 2) + Poly(16, 4))
        self.assertEqual(h5, Poly(120, 1) + Poly(160, 3) + Poly(32, 5))

    def test_chebyshev(self):
        p = self.poly_factory.chebyshev(n=self.N)
        #print p
        #print self.poly_factory.CHEBYSHEV   # sa wszystkie
        self.assertEqual(p.degree(), self.N)
        t0 = self.poly_factory.chebyshev(n=0)
        t1 = self.poly_factory.chebyshev(n=1)
        t2 = self.poly_factory.chebyshev(n=2)
        t3 = self.poly_factory.chebyshev(n=3)
        t4 = self.poly_factory.chebyshev(n=4)
        t5 = self.poly_factory.chebyshev(n=5)
        self.assertEqual(t0, Poly(1))
        self.assertEqual(t1, Poly(1, 1))
        self.assertEqual(t2, Poly(-1) + Poly(2, 2))
        self.assertEqual(t3, Poly(-3, 1) + Poly(4, 3))
        self.assertEqual(t4, Poly(1) + Poly(-8, 2) + Poly(8, 4))
        self.assertEqual(t5, Poly(5, 1) + Poly(-20, 3) + Poly(16, 5))

    def test_legendre(self):
        p = self.poly_factory.legendre(n=self.N)
        #print p
        #print self.poly_factory.LEGENDRE   # sa wszystkie
        self.assertEqual(p.degree(), self.N)
        p0 = self.poly_factory.legendre(n=0)
        p1 = self.poly_factory.legendre(n=1)
        p2 = self.poly_factory.legendre(n=2)
        p3 = self.poly_factory.legendre(n=3)
        p4 = self.poly_factory.legendre(n=4)
        p5 = self.poly_factory.legendre(n=5)
        self.assertEqual(p0, Poly(1))
        self.assertEqual(p1, Poly(1, 1))
        self.assertEqual(p2, Poly(Fraction(-1, 2)) + Poly(Fraction(3, 2), 2))
        self.assertEqual(p3, Poly(Fraction(-3, 2), 1) + Poly(Fraction(5, 2), 3))
        self.assertEqual(p4, Poly(Fraction(3, 8)) 
            + Poly(Fraction(-15, 4), 2) + Poly(Fraction(35, 8), 4))
        self.assertEqual(p5, Poly(Fraction(15, 8), 1) 
            + Poly(Fraction(-35, 4), 3) + Poly(Fraction(63, 8), 5))

    def tearDown(self): pass

if __name__ == "__main__":

    unittest.main()

# EOF



#!/usr/bin/env python3

import itertools
from fractions import Fraction

try:
    rational_types = (int, long, Fraction)
    range = xrange
    from itertools import izip_longest as zip_longest
except NameError:   # Python 3
    rational_types = (int, Fraction)
    from itertools import zip_longest

class Poly(dict):
    """The class defining a poly."""

    def __init__(self, coefficient=0, *arguments):
        """Load up a poly instance."""
        # Tworzymy wielomian c*(x**n0)*(y**n1)*(z**n2)*...
        # Wielomiany maja byc unormowane, bez zer przy wspolczynnikach.
        if coefficient != 0 and arguments:   # normalizacja klucza
            new_key = list(arguments)
            while new_key[-1] == 0 and len(new_key) > 1:
                new_key.pop()
            self[tuple(new_key)] = coefficient
        elif coefficient != 0:
            self[(0,)] = coefficient

    def is_zero(self):
        """Test if the poly is the zero polynomial."""
        return all(self[key] == 0 for key in self) # moze beda zera, bezpieczne
        #return not self   # nie trzymamy zer

    def degree(self):
        """Return the degree of the poly."""
        if self.is_zero():
            return 0
        else:   # jest zabezpieczenie na zerowe wspolczynniki
            return max(sum(key) for key in self if self[key] != 0)

    def cancel(self):
        """Remove all zeros."""
        # Nie moge jednoczesnie iterowac i usuwac kluczy.
        to_delete = [key for key in self if self[key] == 0]
        for key in to_delete:
            del self[key]

    def __repr__(self):
        """Compute the string representation of the poly."""
        self.cancel()
        if self.is_zero():
            return "Poly()"
        else:
            L = list()
            key_list = list(self)
            key_list.sort()
            for key in key_list:
                item = self[key]
                if isinstance(item, Fraction) and item.denominator == 1:
                    # Od razu upraszczamy.
                    item = item.numerator
                    self[key] = item
                tmp = [repr(item)]
                tmp.extend(repr(i) for i in key)
                tmp = ", ".join(tmp)
                L.append("Poly({})".format(tmp))
            return " + ".join(L)

    @classmethod
    def fromiterable(cls, data):
        """Create a poly from a list of coefficients."""
        new_poly = cls()
        for (i, coefficient) in enumerate(data): # wielomian jednej zmiennej
            if coefficient != 0:   # zer nie trzymamy
                new_poly += cls(coefficient, i)
        return new_poly

    def __getitem__(self, key):   # poly[key]
        """Return the coefficient."""
        # Mozemy pytac o dowolny wspolczynnik.
        # user moze podac int zamiast tuple.
        if not isinstance(key, tuple):
            key = (key,)
        # Normujemy klucz.
        new_key = list(key)
        while new_key[-1] == 0 and len(new_key) > 1:
            new_key.pop()
        return self.get(tuple(new_key), 0)

    def __add__(self, other):      # poly1 + poly2
        """Return the sum of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for key in self:
            new_poly[key] = new_poly.get(key, 0) + self[key]
        for key in other:
            new_poly[key] = new_poly.get(key, 0) + other[key]
        new_poly.cancel()
        return new_poly

    __radd__ = __add__

    def __sub__(self, other):       # poly1 - poly2
        """Return the difference of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for key in self:
            new_poly[key] = new_poly.get(key, 0) + self[key]
        for key in other:
            new_poly[key] = new_poly.get(key, 0) - other[key]
        new_poly.cancel()
        return new_poly

    def __rsub__(self, other):       # poly1 - poly2
        """Return the difference of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for key in self:
            new_poly[key] = new_poly.get(key, 0) - self[key]
        for key in other:
            new_poly[key] = new_poly.get(key, 0) + other[key]
        new_poly.cancel()
        return new_poly

    def __eq__(self, other):   # poly1 == poly2
        """Test if polys are equal."""
        return (self - other).is_zero()

    def __ne__(self, other):   # poly1 != poly2
        """Test if polys are not equal."""
        return not self == other

    def __mul__(self, other):        # poly1 * poly2
        """Return the product of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for key1 in self:
            for key2 in other:
                new_key = [x + y for (x, y) in zip_longest(key1, key2, fillvalue=0)]
                new_key = tuple(new_key)
                new_poly[new_key] = ( new_poly.get(new_key, 0) 
                    + self[key1] * other[key2] )
        new_poly.cancel()   # (x-2)*(x+2)=x**2-4, znika x**1
        return new_poly

    __rmul__ = __mul__

    def __pos__(self):
        """Return +poly."""
        return self

    def __neg__(self):
        """Return -poly."""
        new_poly = Poly()
        for key in self:
            new_poly[key] = -self[key]
        return new_poly

    def _power1(self, n):   # poly**n
        new_poly = Poly(1)
        while n > 0:
            new_poly *= self
            n = n - 1
        return new_poly

    def _power3(self, n):       # poly1 ** n, binary
        if n < 0:
            raise ValueError("negative power")
        elif n == 0:
            return Poly(1)
        elif n == 1:
            return self
        elif n == 2:
            return self * self
        else:
            poly = self
            result = Poly(1)    # identycznosc w mnozeniu wielomianow
            while True:
                if n % 2 == 1:
                    result *= poly
                    n = n - 1
                    if n == 0:
                        break
                if n % 2 == 0:
                    poly = poly * poly
                    n = n // 2
            #new_poly.cancel()   # niepotrzebne, bo jest w *
            return result

    __pow__ = _power3

    def __div__(self, other):        # poly / number, monomial / monomial
        """Dividing polys."""
        # Dopuszczamy dzielenie jednomianu przez jednomian.
        if isinstance(other, Poly):
            # Trzeba usunac zerowe wspolczynniki, jezeli byly dopuszczone.
            #self.cancel()
            #other.cancel()
            if len(self) != 1 or len(other) != 1:
                raise ValueError("only monomials can be divided")
            key1 = list(self)[0]
            key2 = list(other)[0]
            new_key = [x - y for (x, y) in
                zip_longest(key1, key2, fillvalue=0)]
            if any(item < 0 for item in new_key):
                raise ValueError("other is greater than self")
            # Normujemy klucz. Przyklad:
            # Poly(1,2,1) / Poly(1,1,1) = Poly(1,1), klucz (1,), a nie (1, 0).
            while new_key[-1] == 0 and len(new_key) > 1:
                new_key.pop()
            new_key = tuple(new_key)
            coefficient1 = self[key1]
            coefficient2 = other[key2]
            if (isinstance(coefficient1, rational_types) and
                isinstance(coefficient2, rational_types)):   # AND
                    # Python 2.7: Fraction(Fraction, Fraction)
                    coefficient = Fraction(coefficient1, coefficient2)
            else:   # float, complex
                coefficient = coefficient1 / coefficient2
            return Poly(coefficient, *new_key)
        elif isinstance(other, rational_types):
            # Python 2.7: Fraction(Fraction, Fraction)
            return self * Poly(Fraction(1, other))
        else:
            return self * Poly(1.0 / other)

    __truediv__ = __div__

    def lcm(self, other):
        """Return the least common multiple of two monomials."""
        if not isinstance(other, Poly):
            other = Poly(other)
        # Jezeli dopuszczamy zera, to trzeba upraszczac.
        #self.cancel()
        #other.cancel()
        if len(self) != 1 or len(other) != 1:
            raise ValueError("only monomials have lcm")
        key1 = list(self)[0]
        key2 = list(other)[0]
        new_key = [max(x, y) for (x, y) in
            zip_longest(key1, key2, fillvalue=0)]
        new_key = tuple(new_key)
        return Poly(1, *new_key)

    def diff(self, var=0):   # rozniczkowanie
        """Return the derivative of the poly."""
        # [c*x**n]' = c*n*x**(n-1)
        new_poly = Poly()
        for key in self:
            if var > len(key)-1 or key[var] == 0:
                continue   # nie ma tej zmiennej w wyrazie
            coefficient = self[key] * key[var]
            new_key = list(key)
            new_key[var] -= 1
            # Normujemy klucz. Przyklad:
            # Poly(1,0,1).diff(1)=1, klucz (0,), a nie (0, 0).
            while new_key[-1] == 0 and len(new_key) > 1:
                new_key.pop()
            new_key = tuple(new_key)
            new_poly[new_key] = coefficient
        return new_poly

    def integrate(self, var=0):   # calkowanie
        """Return the integral of the poly."""
        # integrate(c*x**n, x) = c*x**(n+1)/(n+1) + const
        new_poly = Poly()
        for key in self:
            coefficient = self[key]
            # Trzeba dobrac dlugosc nowego klucza.
            new_key = [0] * max(len(key), var+1)
            # Odtwarzamy stare wartosci klucza.
            for i, item in enumerate(key):
                new_key[i] += item
            if isinstance(coefficient, rational_types):
                coefficient *= Fraction(1, new_key[var] + 1)
            else:
                coefficient /= new_key[var] + 1.0
            # Teraz pojawia sie nowa wartosc klucza.
            new_key[var] += 1
            new_key = tuple(new_key)
            new_poly[new_key] = coefficient
        return new_poly

    def _combine1(self, other, var=0):  # zlozenie wielomianow
        """Return the composition of two polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for key in self:
            coefficient = self[key]
            if var > len(key)-1 or key[var] == 0:
                # Dodajemy wyraz bez zmian.
                new_poly += Poly(coefficient, *key)
                continue
            new_key = list(key)
            new_key[var] = 0   # znika dana zmienna
            new_key = tuple(new_key)
            new_poly += Poly(coefficient, *new_key) * pow(other, key[var])
            # Normowanie jest zawarte w + i *.
        return new_poly

    def _combine2(self, other, var=0):  # zlozenie wielomianow
        """Return the composition of two polys."""
        # Optymalizacja przez przechowywanie juz obliczonych poteg.
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        powers = dict()
        for key in self:
            coefficient = self[key]
            if var > len(key)-1 or key[var] == 0:
                # Dodajemy wyraz bez zmian.
                new_poly += Poly(coefficient, *key)
                continue
            new_key = list(key)
            new_key[var] = 0   # znika dana zmienna
            new_key = tuple(new_key)
            if key[var] not in powers:
                powers[key[var]] = pow(other, key[var])
            new_poly += Poly(coefficient, *new_key) * powers[key[var]]
            # Normowanie jest zawarte w + i *.
        return new_poly

    combine = _combine2

    def key_lex(self):
        """The sorting key for lexicographic order."""
        alist = max([0] + list(key) for key in self)
        return tuple(alist)

    def key_deglex(self):
        """The sorting key for graded lexicographic order."""
        alist = max([sum(key)] + list(key) for key in self)
        return tuple(alist)

    key_deg = key_deglex

    def leading_term(self, key=None):
        """Return the leading term of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            new_key = key(self)
            new_key = tuple(new_key[1:])
            return Poly(self[new_key], *new_key)

    def leading_monomial(self, key=None):
        """Return the leading monomial of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            new_key = key(self)
            new_key = tuple(new_key[1:])
            return Poly(1, *new_key)

    def leading_coefficient(self, key=None):
        """Return the leading coefficient of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            new_key = key(self)
            new_key = tuple(new_key[1:])
            return self[new_key]

    def iterterms(self):
        """The generator for terms from the poly."""
        self.cancel()   # remove zero terms
        for key in self:
            yield Poly(self[key], *key)

# EOF



#!/usr/bin/env python3

import unittest
import random
from fractions import Fraction
from pypolys.mpolys import Poly


class TestPoly(unittest.TestCase):

    def setUp(self):
        self.x = Poly(1, 1)
        self.y = Poly(1, 0, 1)
        self.z = Poly(1, 0, 0, 1)

    def test_init(self):
        self.assertEqual(list(Poly()), [])
        self.assertEqual(list(Poly(2)), [(0,)]) # klucz domyslny
        self.assertEqual(list(Poly(2, 0)), [(0,)])
        self.assertEqual(list(Poly(2, 0, 0, 0)), [(0,)]) # klucz uciety
        self.assertEqual(list(Poly(0, 0, 0, 0)), [])
        self.assertEqual(list(Poly(0, 0, 1, 0)), [])
        self.assertEqual(list(Poly(2, 0, 1, 0)), [(0, 1)]) # klucz uciety

    def test_degree(self):
        self.assertEqual(Poly().degree(), 0)
        self.assertEqual(self.x.degree(), 1)
        self.assertEqual(self.y.degree(), 1)
        p = self.x * self.y
        self.assertEqual(p.degree(), 2)
        p = self.x * self.y * self.z
        self.assertEqual(p.degree(), 3)

    def test_len(self):
        self.assertEqual(len(Poly()), 0)
        self.assertEqual(len(self.x), 1)
        self.assertEqual(len(self.x + self.z), 2)

    def test_repr(self):
        self.assertEqual(repr(Poly()), "Poly()")
        self.assertEqual(repr(Poly(0)), "Poly()")
        self.assertEqual(repr(Poly(0, 0)), "Poly()")
        self.assertEqual(repr(Poly(0, 0, 0)), "Poly()")
        self.assertEqual(repr(Poly(0, 2)), "Poly()")
        self.assertEqual(repr(Poly(0, 2, 3)), "Poly()")
        self.assertEqual(repr(Poly(4)), "Poly(4, 0)")   # tu jest zero
        self.assertEqual(repr(self.x), "Poly(1, 1)")
        self.assertEqual(repr(self.y), "Poly(1, 0, 1)")
        self.assertEqual(repr(self.z), "Poly(1, 0, 0, 1)")
        self.assertEqual(repr(self.x + self.y), "Poly(1, 0, 1) + Poly(1, 1)")
        self.assertEqual(repr(Poly(Fraction(3, 1), 2, 1)), "Poly(3, 2, 1)")
        self.assertEqual(repr(Poly(Fraction(3, 5), 2, 1)), "Poly(Fraction(3, 5), 2, 1)")

    def test_fromiterable(self):
        self.assertEqual(Poly.fromiterable([0, 1]), self.x)
        self.assertEqual(Poly.fromiterable([0, 0, 1]), self.x * self.x)
        self.assertEqual(Poly.fromiterable([0, 1, 1]), self.x + self.x * self.x)

    def test_getitem(self):
        self.assertEqual(self.x[0], 0)   # int
        self.assertEqual(self.x[1], 1)   # int
        self.assertEqual(self.x[0,], 0)   # tuple
        self.assertEqual(self.x[1,], 1)   # tuple
        self.assertEqual(self.x[1, 0], 1)
        self.assertEqual(self.x[0, 1], 0)
        self.assertEqual(self.x[0, 0, 1], 0)
        self.assertEqual(self.y[0, 0], 0)
        self.assertEqual(self.y[0, 1], 1)
        self.assertEqual(self.z[0, 0, 1], 1)
        self.assertEqual(self.z[0, 1, 0], 0)

    def test_add_sub(self):
        self.assertEqual(self.x + self.x, Poly(2, 1))
        self.assertEqual(self.y + self.y, Poly(2, 0, 1))
        self.assertEqual(self.x - self.x, Poly())
        self.assertEqual(self.x + 3, self.x + Poly(3))
        self.assertEqual(3 + self.x, self.x + Poly(3))
        self.assertEqual(self.x - 3, self.x - Poly(3))
        self.assertEqual(3 - self.x, Poly(3) - self.x)

    def test_mul(self):
        self.assertEqual(self.x * self.x, Poly(1, 2))
        self.assertEqual(self.y * self.y, Poly(1, 0, 2))
        self.assertEqual(self.z * self.z, Poly(1, 0, 0, 2))
        self.assertEqual(self.x * self.y, Poly(1, 1, 1))
        self.assertEqual(self.x * self.z, Poly(1, 1, 0, 1))
        self.assertEqual(self.y * self.z, Poly(1, 0, 1, 1))
        self.assertEqual(self.x * self.y * self.z, Poly(1, 1, 1, 1))
        self.assertEqual(self.y * 3, Poly(3, 0, 1))
        self.assertEqual(3 * self.z, Poly(3, 0, 0, 1))

    def test_pos_neg(self):
        self.assertEqual(+self.x, self.x)
        self.assertEqual(-self.x, Poly(-1, 1))
        self.assertEqual(-self.y, Poly(-1, 0, 1))

    def test_pow(self):
        self.assertEqual(self.x ** 3, Poly(1, 3))
        self.assertEqual(self.y ** 5, Poly(1, 0, 5))
        p = self.x + Poly(1)
        self.assertEqual(p ** 3, p*p*p)
        self.assertEqual(p ** 5, p*p*p*p*p)
        p = self.y + Poly(2)
        self.assertEqual(p ** 3, p*p*p)

    def test_cancel(self):
        p = pow(self.x + self.y, 2) - Poly(2) * self.x * self.y
        self.assertEqual(len(p), 2)
        p = self.x ** 3 + Poly(-1, 3)
        self.assertEqual(len(p), 0)

    def test_div(self):
        self.assertEqual(repr(self.x / 2), "Poly(Fraction(1, 2), 1)")
        self.assertEqual(self.x / 2, Poly(Fraction(1, 2), 1))
        self.assertEqual(repr(self.x / 2.0), "Poly(0.5, 1)")
        self.assertEqual(repr(self.y / 2), "Poly(Fraction(1, 2), 0, 1)")
        self.assertEqual(repr(self.y / 2.0), "Poly(0.5, 0, 1)")
        self.assertEqual(Poly(6, 2, 3, 4) / Poly(2, 1, 2), Poly(3, 1, 1, 4))
        self.assertEqual(Poly(3, 2, 1) / Poly(2, 1, 1), Poly(Fraction(3, 2), 1))
        self.assertRaises(ValueError, lambda: self.x / self.y)
        self.assertRaises(ValueError, lambda: (self.x + self.z) / self.y)
        self.assertRaises(ValueError, lambda: self.x / (self.y + self.z))

    def test_lcm(self):
        self.assertEqual(self.x.lcm(1), self.x)
        self.assertEqual(self.x.lcm(self.y), self.x * self.y)
        self.assertEqual(self.z.lcm(self.y), self.z * self.y)
        self.assertRaises(ValueError, lambda: self.z.lcm(self.x + self.y))
        self.assertRaises(ValueError, lambda: (self.x + self.y).lcm(self.z))

    def test_diff(self):
        self.assertEqual(self.x.diff(2), Poly())
        self.assertEqual(self.z.diff(0), Poly())
        self.assertEqual(self.x.diff(0), Poly(1))
        self.assertEqual(self.y.diff(1), Poly(1))
        self.assertEqual(self.z.diff(2), Poly(1))
        p = Poly(3, 2, 5)
        self.assertEqual(p.diff(0), Poly(6, 1, 5))
        self.assertEqual(p.diff(1), Poly(15, 2, 4))
        self.assertEqual(p.diff(2), Poly())

    def test_integrate(self):
        self.assertEqual(self.x.integrate(0), Poly(Fraction(1, 2), 2))
        self.assertEqual(self.x.integrate(1), self.x * self.y)
        self.assertEqual(self.y.integrate(2), self.y * self.z)
        self.assertEqual(Poly(8, 3).integrate(0), Poly(2, 4))
        self.assertEqual(Poly(3, 2, 4).integrate(1), Poly(Fraction(3, 5), 2, 5))
        self.assertEqual(Poly(3, 0, 4).integrate(0), Poly(3, 1, 4))
        self.assertEqual(Poly(Fraction(3, 2), 1, 6, 4).integrate(2),
            Poly(Fraction(3, 10), 1, 6, 5))
        self.assertEqual(Poly(Fraction(3, 2), 1, 1).integrate(1),
            Poly(Fraction(3, 4), 1, 2))
        self.assertEqual(Poly(1.5, 1).integrate(0), Poly(0.75, 2))

    def test_combine(self):
        self.assertEqual(Poly(1, 3).combine(Poly(1, 2), var=0), Poly(1, 6))
        self.assertEqual((self.x ** 3).combine(self.y ** 2, var=0), self.y ** 6)
        self.assertEqual(self.x.combine(self.y ** 2, var=0), self.y ** 2)
        self.assertEqual(self.x.combine(self.y, var=2), self.x)
        self.assertEqual((self.x ** 2).combine(self.z + 3 * self.y, var=0), 
            self.z ** 2 + 6 * self.z * self.y + 9 * self.y ** 2)
        self.assertEqual((self.x ** 2 + self.y).combine(self.x ** 2, var=1),
            2 * self.x ** 2)
        # combine() wlasciwie realizuje eval().
        self.assertEqual(Poly(1, 3).combine(2, var=0), 8)
        self.assertEqual(Poly(1, 0, 3).combine(2, var=1), 8)
        self.assertEqual((Poly(3, 1, 2) + Poly(5, 0, 2, 1)).combine(
            2 * self.x, var=1), Poly(12, 3) + Poly(20, 2, 0, 1))

    def test_sort(self):
        lex_list = [Poly(1), self.z, self.z ** 2, 
            self.y, self.y * self.z, self.y **2, 
            self.x, self.x * self.z, self.x * self.y, self.x ** 2]
        deglex_list = [Poly(1), self.z, self.y, self.x,
            self.z ** 2, self.y * self.z, self.y ** 2,
            self.x * self.z, self.x * self.y, self.x ** 2]
        blist = list(deglex_list)
        random.shuffle(blist)
        # Sortowanie wielomianow.
        blist.sort(key=Poly.degree)
        self.assertEqual(blist[0].degree(), 0)
        self.assertEqual(blist[1].degree(), 1)
        self.assertEqual(blist[2].degree(), 1)
        self.assertEqual(blist[3].degree(), 1)
        self.assertEqual(blist[4].degree(), 2)
        blist.sort(key=Poly.key_lex)
        self.assertEqual(lex_list, blist)
        blist.sort(key=Poly.key_deglex)
        self.assertEqual(deglex_list, blist)

    def test_leading_term(self):
        p1 = pow(3 * self.x + 1, 2)
        self.assertEqual(p1.leading_term(key=Poly.key_lex), 9 * self.x ** 2)
        self.assertEqual(p1.leading_monomial(key=Poly.key_lex), self.x ** 2)
        self.assertEqual(p1.leading_coefficient(key=Poly.key_lex), 9)
        p1 = pow(3 * self.y + 1, 2)
        self.assertEqual(p1.leading_term(key=Poly.key_lex), 9 * self.y ** 2)
        self.assertEqual(p1.leading_monomial(key=Poly.key_lex), self.y ** 2)
        self.assertEqual(p1.leading_coefficient(key=Poly.key_lex), 9)
        p1 = pow(2 * self.x + self.y, 3)
        self.assertEqual(p1.leading_term(key=Poly.key_lex), 8 * self.x ** 3)
        self.assertEqual(p1.leading_monomial(key=Poly.key_lex), self.x ** 3)
        self.assertEqual(p1.leading_coefficient(key=Poly.key_lex), 8)
        self.assertEqual(Poly(5).leading_term(key=Poly.key_lex), Poly(5))
        self.assertEqual(Poly(5).leading_monomial(key=Poly.key_lex), Poly(1))
        self.assertEqual(Poly(5).leading_coefficient(key=Poly.key_lex), 5)
        self.assertRaises(ValueError, Poly().leading_term, Poly.key_lex)
        self.assertRaises(ValueError, Poly().leading_monomial, Poly.key_lex)
        self.assertRaises(ValueError, Poly().leading_coefficient, Poly.key_lex)

    def test_iterterms(self):
        p1 = pow(self.x * self.y + 1, 2)
        #print list(p1.iterterms())
        self.assertEqual(len(list(p1.iterterms())), 3)
        self.assertEqual(set(term.degree() for term in p1.iterterms()), set([0, 2, 4]))
        self.assertEqual(sum(term.degree() for term in p1.iterterms()), 6)

    def tearDown(self): pass

if __name__== "__main__":

    unittest.main()

# EOF



#!/usr/bin/env python3

from fractions import Fraction

try:
    rational_types = (int, long, Fraction)
    range = xrange
except NameError:   # Python 3
    rational_types = (int, Fraction)


class Poly(dict):
    """The class defining a poly."""

    def __init__(self, coefficient=0, n=0):
        """Load up a poly instance."""
        # Na bazie Sedgewicka - tworzymy wielomian c*(x**n).
        # Wielomiany maja byc unormowane, bez zer przy wspolczynnikach.
        if coefficient != 0:
            self[n] = coefficient

    def is_zero(self):
        """Test if the poly is the zero polynomial."""
        # Pierwszy sposob dziala tez dla zerowych wspolczynnikow.
        # To jest szybkie, bo pierwszy niezerowy konczy petle.
        return all(self[k] == 0 for k in self)
        # Drugi sposob - gdy zer nie trzymamy.
        #return not self

    def degree(self):
        """Return the degree of the poly."""
        if self.is_zero():
            return 0
        else:   # jest zabezpieczenie na zerowe wspolczynniki
            return max(k for k in self if self[k] != 0)

    key_deg = degree

    key_lex = degree

    key_deglex = degree

    def cancel(self):
        """Remove all zeros."""
        # Nie moge jednoczesnie iterowac i usuwac kluczy.
        to_delete = [k for k in self if self[k] == 0]
        for k in to_delete:
            del self[k]

    def __repr__(self):
        """Compute the string representation of the poly."""
        self.cancel()
        if self.is_zero():
            return "Poly()"
        else:
            L = list()
            key_list = list(self)
            key_list.sort()
            for k in key_list:
                item = self[k]
                if isinstance(item, Fraction) and item.denominator == 1:
                    # Od razu upraszczamy. To wolno podczas iteracji.
                    item = item.numerator
                    self[k] = item
                if k == 0:
                    L.append("Poly({})".format(repr(item)))
                else:
                    L.append("Poly({}, {})".format(repr(item), k))
            return " + ".join(L)

    @classmethod
    def fromiterable(cls, data):
        """Create a poly from coefficients."""
        new_poly = cls()
        for (k, coefficient) in enumerate(data):
            if coefficient != 0:   # zer nie trzymamy
                new_poly[k] = coefficient
        return new_poly

    def __getitem__(self, k):   # poly[k]
        """Return the coefficient."""
        # Mozemy pytac o dowolnie duzy wspolczynnik.
        return self.get(k, 0)

    def __add__(self, other):   # poly1 + poly2, poly + number, number + poly
        """Return the sum of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for k in self:
            new_poly[k] = new_poly.get(k, 0) + self[k]
        for k in other:
            new_poly[k] = new_poly.get(k, 0) + other[k]
        # To moze zwolnic kod.
        #new_poly.cancel()   # moze byc x + (-x) = 0
        return new_poly

    __radd__ = __add__

    def __sub__(self, other):       # poly1 - poly2
        """Return the difference of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for k in self:
            new_poly[k] = new_poly.get(k, 0) + self[k]
        for k in other:
            new_poly[k] = new_poly.get(k, 0) - other[k]
        # To moze zwolnic kod.
        #new_poly.cancel()   # moze byc x - x = 0
        return new_poly

    def __rsub__(self, other):       # poly1 - poly2
        """Return the difference of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for k in self:
            new_poly[k] = new_poly.get(k, 0) - self[k]
        for k in other:
            new_poly[k] = new_poly.get(k, 0) + other[k]
        # To moze zwolnic kod.
        #new_poly.cancel()
        return new_poly

    def __eq__(self, other):   # poly1 == poly2
        """Test if polys are equal."""
        return (self - other).is_zero()

    def __ne__(self, other):   # poly1 != poly2
        """Test if polys are not equal."""
        return not self == other

    def __mul__(self, other):        # poly1 * poly2
        """Return the product of polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        new_poly = Poly()
        for i in self:
            for j in other:
                new_poly[i+j] = new_poly.get(i+j, 0) + self[i] * other[j]
        # To moze zwolnic kod.
        #new_poly.cancel()  # (x-2)*(x+2)=x**2-4, znika x**1
        return new_poly

    __rmul__ = __mul__

    def __pos__(self):
        """Return +poly."""
        return self

    def __neg__(self):
        """Return -poly."""
        new_poly = Poly()
        for k in self:
            new_poly[k] = -self[k]
        return new_poly

    def _eval1(self, x):   # schemat Hornera
        k = self.degree()
        result = self[k]   # istnieje
        while k > 0:
            k = k - 1
            result = result * x + self.get(k, 0)
        return result

    eval = _eval1

    def _combine1(self, other):  # zlozenie funkcji/wielomianow
        """Return the composition of two polys."""
        if not isinstance(other, Poly):
            other = Poly(other)
        k = self.degree()         # tez schemat Hornera
        new_poly = Poly(self[k])
        while k > 0:
            k = k - 1
            new_poly = new_poly * other + Poly(self.get(k, 0))
        #new_poly.cancel()   # niepotrzebne, bo jest w + i *
        return new_poly

    combine = _combine1

    def _power1(self, n):   # poly1 ** n
        new_poly = Poly(1)
        while n > 0:
            new_poly *= self
            n = n - 1
        #new_poly.cancel()   # niepotrzebne, bo jest w *
        return new_poly

    def _power2(self, n):     # poly1 ** n
        return Poly(1, n).combine(self)

    def _power3(self, n):       # poly1 ** n, binary
        if n < 0:
            raise ValueError("negative power")
        elif n == 0:
            return Poly(1)
        elif n == 1:
            return self
        elif n == 2:
            return self * self
        else:
            poly = self
            result = Poly(1)    # identycznosc w mnozeniu wielomianow
            while True:
                if n % 2 == 1:
                    result *= poly
                    n = n - 1
                    if n == 0:
                        break
                if n % 2 == 0:
                    poly = poly * poly
                    n = n // 2
            #new_poly.cancel()   # niepotrzebne, bo jest w *
            return result

    __pow__ = _power3

    def diff(self):   # rozniczkowanie
        """Return the derivative of the poly."""
        # [c*x**n]' = c*n*x**(n-1)
        new_poly = Poly()
        for k in self:
            if k > 0:
                new_poly[k-1] = k * self[k]
        # Normowanie nie jest potrzebne.
        return new_poly

    def integrate(self):   # calkowanie
        """Return the integral of the poly."""
        # integrate(c*x**n, x) = c*x**(n+1)/(n+1) + const
        new_poly = Poly()
        for k in self:
            if isinstance(self[k], rational_types):
                new_poly[k+1] = self[k] * Fraction(1, k+1)
            else:
                new_poly[k+1] = self[k] / (k+1.0)
        # Normowanie nie jest potrzebne.
        return new_poly

    def __div__(self, other):        # poly / number, monomial / monomial
        """Dividing polys."""
        # Dopuszczamy dzielenie jednomianu przez jednomian.
        if isinstance(other, Poly):
            # Trzeba usunac zerowe wspolczynniki.
            self.cancel()
            other.cancel()
            if len(self) != 1 or len(other) != 1:
                raise ValueError("only monomials can be divided")
            k1 = list(self)[0]
            k2 = list(other)[0]
            if k1 < k2:
                raise ValueError("other is greater than self")
            coefficient1 = self[k1]
            coefficient2 = other[k2]
            if (isinstance(coefficient1, rational_types) and
                isinstance(coefficient2, rational_types)):   # AND
                    # Python 2.7: Fraction(Fraction, Fraction)
                    coefficient = Fraction(coefficient1, coefficient2)
            else:   # float, complex
                coefficient = coefficient1 / coefficient2
            return Poly(coefficient, k1 - k2)
        # Python 2.7: Fraction(Fraction, Fraction)
        elif isinstance(other, rational_types):
            return self * Poly(Fraction(1, other))
        else:
            return self * Poly(1.0 / other)

    __truediv__ = __div__

    def lcm(self, other):
        """Return the least common multiple of two monomials."""
        if not isinstance(other, Poly):
            other = Poly(other)
        self.cancel()
        other.cancel()
        if len(self) != 1 or len(other) != 1:
            raise ValueError("only monomials have lcm")
        k1 = list(self)[0]
        k2 = list(other)[0]
        return Poly(1, max(k1, k2))

    def __call__(self, x):
        if isinstance(x, Poly):
            return self.combine(x)
        else:
            return self.eval(x)

    def leading_term(self, key=None):
        """Return the leading term of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            k_max = key(self)
            return Poly(self[k_max], k_max)

    def leading_monomial(self, key=None):
        """Return the leading monomial of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            k_max = key(self)
            return Poly(1, k_max)

    def leading_coefficient(self, key=None):
        """Return the leading coefficient of the poly."""
        if self.is_zero():
            raise ValueError("zero poly")
        else:
            k_max = key(self)
            return self[k_max]

    def iterterms(self):
        """The generator for terms from the poly."""
        self.cancel()   # remove zero terms
        for k in self:
            yield Poly(self[k], k)

# EOF



#!/usr/bin/env python3

import unittest
import random
from fractions import Fraction
from pypolys.polys import Poly

x = Poly(1, 1)
p = (x-1) * (x+1) +1
print(p)   # tu jest p.cancel() w __repr__
print(len(p))   # ogolnie moga byc zerowe wspolczynniki
assert (p / x) == x   # trzeba usunac zerowe wspolczynniki

# EOF
